<?php

/**
 * $Id: carbon_chart.inc,v 1.5 2007/02/08 07:54:05 johnackers Exp $ 
 * 
 * chart functions for drupal carbon module. 
 * 
 * Created on 29-jan-2006
 */

/**
 * append links for available charts for this data
 */

function carbon_chart_link(&$links)
{
  $query = $_GET['q'];    	
  $links[] = l("bar chart", $query, null, "chart=bar") ;   
  $links[] = l("pie chart", $query, null, "chart=pie") ;   
}

/**
 * if the user has requested a chart in the query 
 * embed the IMG tags into the HTML  
 */

function carbon_chart_embed($start)
{  
  if (isset($_GET['chart']))
  { 
    $query = $_GET['q'];
    $chart = $_GET['chart'];
    $link = "?q=".$query."&render=".$chart ; 
    if (isset($start))
        $link .= "&start=".$start ;    
    return "<a href='".$link."'><img src='".$link."' alt='".$chart."' style='margin:10px'></a>" ; 
  }   	  
}


/**
 * Is this a request to render a particular chart
 */

function carbon_chart_requested()
{
  return isset($_GET['render']);
}

/**
 * get the start date which should be embedded in request
 */

function carbon_chart_get_startdate()
{
  return ($_GET['start']);
}


/**
 * generate an image that contains the chart that had been  
 * embedded in the HTML
 * 
 * @param an array containing multiple $key => $values where key 
 * is the name of the item and value is size of the item 
 */


function carbon_chart_generate($map, $title)
{
//  $map = array("abc" => 344, "def" => 432);

  $chart = $_GET['render'];  
  // make up the function name based on the render request
  $fn = "carbon_".$chart."_chart"; 
  if (function_exists($fn))
  {
    return $fn($map, $title);
  }
  else 
    return("This function does not exist: ". $fn);   
}


/**
 * create a bar chart as a png/image
 * @param an array of $title => $amount
 */


function carbon_bar_chart($dataset, $title)
{
  // allow the chart to create a large vertical (scrollable) image
  carbon_bar_chart_horiz($dataset, $title, 100, 100, 480, 10000);
}


/**
 * bars go left to right
 * 
 */


function carbon_bar_chart_horiz($dataset, $title, $min_width, $min_height, $max_width, $max_height)
{
  global $customPalette; 
    
  $labelWidth = 140 ; //  large enough to accomodate the key text
  $barWidth = 30 ;    // start with arbitrary width  
  $spacePreLabel = 10 ; 
  // work out sizes 
  $width = $barWidth * count($dataset);
  
  $labelHeight = 20 ;  // the sector logos 
   
  $barMaxHeight = $max_height - $labelHeight ; 
  $barMinHeight = $min_height - $labelHeight ; 
    
  if (empty($dataset))
    return "empty array";     

  // adjust the bar width is the height is very small
       
  if ($width < $barMinHeight) 
    $barWidth = $barMinHeight / count($dataset); 
    
  // of the height is enormous, actually would be 
  // better to stop adding images.
     
  if ($width > $barMaxHeight) 
    $barWidth = $barMaxHeight / count($dataset);     

  
  $imgHeight = $barWidth * count($dataset) + $labelHeight; 
  $imgWidth = $max_width ; 
    
  header("Content-type: image/png");

// Create image and define colors

  $image=imagecreate($imgWidth, $imgHeight);

  $colNext = 0 ;  
  
  $colorWhite=imagecolorallocate($image, 255, 255, 255); 
  $colorBase=imagecolorallocate($image, 128, 128, 128);

  $shadow = false ;

  $baseWidth = 2 ;   
  $baseLength = $barWidth * count($dataset); 
  
  $shadow = 2 ; 
  $bareBarWidth = $barWidth - $shadow ; 
   
  $barMaxLength = $imgWidth - $baseWidth - $labelWidth; 
    
  $max_value = 0 ; 
  foreach($dataset as $key => $value)
  {   
    if (is_array($value))     
  	  $subtotal = array_sum($value);
    else
      $subtotal = $value ; 
    $max_value = max($max_value, $subtotal);  
  }  
  $scaling = $max_value == 0 ? 1 : ($barMaxLength)/$max_value;


  // put down a base line  
  
  imagefilledrectangle($image, 0,  0, $imgWidth, $imgHeight, $colorWhite);

  imagefilledrectangle($image, 0,  0, $baseWidth, $baseLength, $colorBase);

  // Create bar charts

  $barNum = 0 ;
  foreach ($dataset as $outerKey => $outerValue)
  { 
    $sum = 0 ;
  	$innerDataset = null ;           
    if (is_array($outerValue))
    {
      $innerDataset = $outerValue;
      $sum = array_sum($innerDataset);    
    }
    else
    {
      // create an inner dataset of size 1         
      $innerDataset = array();
      $innerDataset[$outerKey] = $sum = $outerValue ;        	
    }
    //
    // write a set of bars, one extending the other
    // 
    $x = $baseWidth;
    $y = $barNum*$barWidth; 
    create_palette($innerDataset, $image);
            	
    foreach ($customPalette as $key=>$col) 
    {    	      
      if (!isset($innerDataset[$key]))
        continue ; 
      $value = $innerDataset[$key];
      $barLength = $value * $scaling ; 

      // right_side, top, left_side, bottom

      imagefilledrectangle($image, $x, $y, $x += $barLength, $y+$bareBarWidth, $col);
      $x += 2 ;   // leave a gap between the bars

 //     if ($shadow)
 //       imagefilledrectangle($image, $baseWidth, $y = $i*$barWidth+$bareBarWidth, $baseWidth+$barLength-3,  $y += $shadow, $darkgray);
        
    }
    imagestring($image, 2,  $x+$spacePreLabel, $y + (0.5*$barWidth) - 6, $outerKey." - ".sprintf("%.0f", $sum), 255);
    $barNum++ ; 
  }
  // Output graph and clear image from memory
  
  $labelx = 10 ; 
  $y += $barWidth + 5 ; 
  foreach ($customPalette as $name => $col)
  {
    imagefilledrectangle($image, $labelx, $y, $labelx+10, $y+10, $col) ;   
    imagestring($image, 2, $labelx + 15, $y, $name, 255);
    $labelx += strlen($name) * 13 ; 
  }     
  imagepng($image); 
  imagedestroy($image);    
}



function carbon_bar_chart_vert($dataset, $title, $min_width, $min_height, $max_width, $max_height)
{
  $labelWidth = 200 ; //  large enough to accomodate the key text
  $barWidth = 40 ;    // start with arbitrary width  
  
  // work out sizes 
  $width = $barWidth * count($dataset);
  if ($width < $min_width) 
    $barWidth = $min_width / count($dataset); 
  if ($width > $max_width) 
    $barWidth = $max_width / count($dataset);     
  
  $imgWidth = $barWidth * count($dataset); 
  $imgHeight = $max_height ; 
    
  header("Content-type: image/png");

// Create image and define colors

  $image=imagecreate($imgWidth, $imgHeight);
  $colorWhite=imagecolorallocate($image, 255, 255, 255);
  $colorGrey=imagecolorallocate($image, 192, 192, 192);
  $colorBase=imagecolorallocate($image, 128, 128, 128);

  $colorFG=imagecolorallocate($image, 64, 64, 64);
  $colorBG=imagecolorallocate($image, 192, 192, 192);

  $baseHeight = 2 ;   
  $bareBarWidth = $barWidth ; 
  $barBottom = $imgHeight - $baseHeight - 1;
  $barTop = $labelWidth ; 
  
  $scaling = max($dataset) == 0 ? 1 : ($barBottom - $barTop)/max($dataset);

  // put down a base line  

  imagefilledrectangle($image, 0, $imgHeight -$baseHeight, $imgWidth, $imgHeight, $colorBase);

  // Create bar charts

  $i = 0 ;
  foreach ($dataset as $key => $value){
            
    $barHeight = $value * $scaling ;   
    // right_side, top, left_side, bottom
    imagefilledrectangle($image, $i*$bareBarWidth, $barBottom-$barHeight, ($i+1)*$bareBarWidth, $barBottom, $colorFG);
    imagefilledrectangle($image, ($i*$bareBarWidth)+1, $barBottom-$barHeight+1, (($i+1)*$bareBarWidth)-5, $barBottom, $colorBG);
    imagestringup($image, 2, $i*$bareBarWidth + $bareBarWidth/2 - 10, $barBottom-$barHeight-10, $key." - ".sprintf("%.0f", $value), 255);
    $i++ ;
  }
  // Output graph and clear image from memory
  
  imagepng($image); 
  imagedestroy($image);    
}



/**
 * 
 * create a pie chart as a png/image
 * @param an array of $title => $amount
 */


function carbon_pie_chart($outerset, $toptitle)
{
  global $white, $black, $lightgray ; 
  
  $charts = array();
  $imagex = 0 ; 
  $imagey = 0 ; 
  $x = 0 ; 
  $y = 0 ;
  $maxwidth = 480 ;

  $y = 40 ; // make room for title 
  $i = 0 ;
  foreach ($outerset as $title => $dataset)
  {
    $chart = new PieChart();  
    $legend = new Legend(); 

    // get the sizeof the image and legend
    $chart->size($title, $dataset); 
    $legend->size($dataset);
    
    if ($x + max($chart->sizex, $legend->sizex) < $maxwidth)
    {
      $chart->posx = $legend->posx = $x ;
      $chart->posy = $y ; 
      $legend->posy = $y + $chart->sizey;            ; 
    } 
    else  // move to the next line
    {
      $chart->posx = $legend->posx = $x = 0 ;     
      $chart->posy = $y = $imagey;
      $legend->posy = $y + $chart->sizey; 
    }
    $x += 10 + max($chart->sizex, $legend->sizex);
          
    $imagex = max($imagex, $x + $chart->sizex); 
    $imagey = max($imagey, $legend->posy + $legend->sizey);
    
    $charts[] = $chart ; 
    $charts[] = $legend ;   
  }
  
//  $imagey = $y ;
    
  $image = imagecreate($imagex+1, $imagey+1); 
  
  define_palette($image); 
  imagefilledrectangle($image, 0, 0, $imagex, $imagey, $white) ;

  imagestring($image, 5, 20, 15, $toptitle, $black);  
  
  $i = 0 ; 
  foreach ($charts as $chart)
  {    
  // debug - draw an outline around the box we are going to fill 
    imagerectangle($image, $chart->posx, $chart->posy, 
      $chart->posx+$chart->sizex, $chart->posy+$chart->sizey, $lightgray) ;
    $chart->render($image); 
  }
  
  
  // flush image
  header('Content-type: image/png');
  imagepng($image);
  imagedestroy($image);
} 

/**
 * create a basic color palette to use with pie charts 
 * (this may not be necessary, it seems everyone would need
 * to do this)
 */


function define_palette($image)
{
// allocate the CGA 16 color palette
  global $standardPalette, $sparePalette, $black, $white, $lightgray ;
  if (isset($standardPalette) && isset($sparePalette))
    return ;
    
  $black    = imagecolorallocate($image, 0x00, 0x00, 0x00);
  $blue     = imagecolorallocate($image, 0x00, 0x00, 0xAA);
  $green    = imagecolorallocate($image, 0x00, 0xAA, 0x00);
  $cyan     = imagecolorallocate($image, 0x00, 0xAA, 0xAA);
  $red      = imagecolorallocate($image, 0xAA, 0x00, 0x00);         
  $magenta  = imagecolorallocate($image, 0xAA, 0x00, 0xAA);
  $brown    = imagecolorallocate($image, 0xAA, 0x55, 0x00);
  $lightgray= imagecolorallocate($image, 0xF8, 0xF8, 0xF8);
  $darkgray = imagecolorallocate($image, 0x55, 0x55, 0x55);  
  $b_blue   = imagecolorallocate($image, 0x55, 0x55, 0xFF);
  $b_green  = imagecolorallocate($image, 0x55, 0xFF, 0x55);
  $b_cyan   = imagecolorallocate($image, 0x55, 0xFF, 0xFF);
  $b_red    = imagecolorallocate($image, 0xFF, 0x55, 0x55);  
  $b_magenta= imagecolorallocate($image, 0xFF, 0x55, 0xFF);
  $yellow   = imagecolorallocate($image, 0xFF, 0xFF, 0x55); 
  $white    = imagecolorallocate($image, 0xFF, 0xFF, 0xFF);


  $sparePalette = array(
               $brown, $cyan, 
               $b_red, $b_green, $b_blue,  
               $b_magenta,  $yellow, $b_cyan, $black);
        
  $standardPalette = array(
        "heating" => $red, 
        "electricity" => $blue, 
        "transport" => $green, 
        "car" => $magenta, 
        "aviation" => $b_blue);
}

/**
 * create a custom palette to match the keys provided in the 
 * dataset. Where keys match known keys, use a preferred colour.
 * Then use colours from the spare palette. 
 * 
 * Once all the colours are used, don't show anything else.
 */


function create_palette($dataset, $image)
{
	global $standardPalette, $sparePalette, $customPalette;
     
  define_palette($image);

  if (!isset($customPalette))
    $customPalette = array();
    
  $keys_not_assigned = array_merge($dataset);

  // have already allocated a colour for this key
  
  foreach($customPalette as $key => $col)
  {
    if (isset($keys_not_assigned[$key]))
    {
      unset($keys_not_assigned[$key]);
    }
  }
  
  foreach($standardPalette as $key => $col)
  {
    if (isset($keys_not_assigned[$key]))
    {
    	$customPalette[$key] = $standardPalette[$key];
      unset($keys_not_assigned[$key]);
    }
  } 

  $i = 0 ;  foreach($keys_not_assigned as $key => $object)
  {
    while ($i < count($sparePalette))
    {
    	$col = $sparePalette[$i++];
      if (in_array($col,$customPalette))
        continue;   
      $customPalette[$key] = $col ;  
      unset($keys_not_assigned[$key]);
    }
  }     
}

/** 
 * It is very painful using objects in PHP that has to run under version 4 PHP. 
 * To difference between 4 and 5 as described by others is that 4 passes objects
 * by value and 5 copies objects by reference. 
 * 
 * However it is very useful when you rendering different shapes of objects to 
 * use objects. There are two methods used to display objects
 *
 * This class draws a pie chart.
 *  
 * size() - allow the object to work out how much space it will take up on 
 * the canvas
 * 
 * render() - draw the object on the canvas.
 *  
 */


class PieChart 
{
  var $sizex, $sizey ;       // calculated when size() invoked. 
  var $posx, $posy  ;        // set by external functions to postion object
  var $title, $titleheight ;
  var $dataset ; 
  var $ak ;
  var $radius = 50 ;
  
  function size(&$title, &$dataset)
  {
    $this->title = $title; 
    $this->dataset = $dataset; 
    // create image
    $this->ak = array_keys($dataset);
    $this->titleheight = 30 ; 
    $this->sizey =  $this->titleheight + $this->radius * 2 ; 	
    $this->sizex =  $this->radius * 4 ; 
  }	    
  
  function render(&$image)
  {
    global $customPalette, $black; 
    $total = 0 ; 
    
    // write the title 
    imagestring($image, 3, $this->posx + 20, $this->posy + $this->titleheight/3, $this->title, $black);
    
    // work out how many degrees each piece of the 
    // pie can be, making a total of 360 degrees. 
   
    foreach ($this->dataset as $key => $value)
    {
      $total += $value ;
    }   
    $scaling = ($total == 0) ? 0 : 360/$total;     
    
    $i = 0 ;
    $last = 0 ; 
  
//    imagestring($image, 3, 20, 20, $this->ak[0], $black);       
  
    create_palette($this->dataset, $image);
                
    foreach ($customPalette as $key=>$col)  
    {   
      if (!isset($this->dataset[$key]))
        continue ; 
      $value = $this->dataset[$key];
      $next = $value * $scaling + $last;


  
      //  draw the pie 
      //  cx, cy, w, h, s, e, col, 
      imagefilledarc($image, 
        $this->posx + 2*$this->radius, $this->posy + $this->titleheight + $this->radius, 
        4*$this->radius, 2*$this->radius, 
        $last, $next, 
        $col, IMG_ARC_PIE);
          
      $last = $next ;    $i++ ; 
    }        	
  }      
}

/** 
 * This class writes the legend for a pie chart or a bar chart
 */


class Legend 
{
  var $sizex = 0, $sizey = 0 ; 
  var $posx, $posy	;

  var $legend_height ; 
  var $dataset ; 
  
  function size(&$dataset)
  {
    global $customPalette;
    $this->dataset = $dataset ; 
    $this->legend_height = 18 ; 
    $this->sizex = 150 ;

    foreach ($dataset as $key=>$val)  
    {   
      if (empty($val)) 
        continue ;
      $this->sizey += $this->legend_height;
    }
  }

  function render(&$image)
  {
    global $customPalette, $black;
    $i = 0 ;  
    foreach ($customPalette as $key=>$col)  
    {   
    	if (empty($this->dataset[$key])) 
        continue ; 
      $y = $this->posy + ($i * $this->legend_height) ; 
  
      // draw the key 
      imagefilledrectangle($image, $this->posx, $y, $this->posx+10, $y+10, $col);
      // 
      // now write the entry into the keys. 
      //  
      imagestring($image, 2, $this->posx + 20, $y, $key." - ".sprintf("%.0f", $this->dataset[$key]), $black);       
      $i++ ; 
    }  	
  }  
}




?>